
/****************************************************
	MARC Template Library 

	Version 0.6.1 - Released 12 June 2002
    Version 0.6 - Released 10 June 2002
	  Version 0.5 - Not Released 1st June 2002
    Version 0.4.5 - Released 5th April 2002
	Version 0.4 - Released 28th March 2002
  Version 0.3 - Released 18th January 2002
  Version 0.2 - Released 14th January 2002
  Version 0.1 - Released 8th January 2002


	Copyright Mark Basedow 2002.
	Email: markbasedow@bigpond.com

	Please read licence.txt that is
	distributed with this file.

  ***************************************************/

#ifndef _MARCBUFFER_H
#define _MARCBUFFER_H

#include <string>
#include <fstream>
#include <iomanip>
#include <iostream>
#include <string>
#include "marcutil.h"
//#include "marceventrecord.h"

using namespace std;


/*
MARCBuffer
MARCFileBuffer

  */

template<class TRecType>
class MARCBuffer 
{
public:
	MARCBuffer();
	virtual ~MARCBuffer();
	MARCBuffer(const MARCBuffer & mb);

	void Reset();
	void Clear();

	MARCBuffer & operator=(const MARCBuffer & mb);
	MARCBuffer operator+(const MARCBuffer & mb);

	bool LoadFromFile(const char * szFileName);
	bool LoadFromMem(const char * buffer, long lSize);

	bool Read(TRecType & rec);

//	long NumberOfRecords();

	/*
	ReadFromFile is called first with a filename to open the file and return first file
	subsequent calls use only the Record param
  */
	bool ReadFromFile(TRecType & rec, char * szFileName = NULL, long lBufSize = 10000); 

	const char * GetBuffer() { return buf; }
	long GetBufferLength() { return lLength; }

protected:
	char * buf;
	long lLength;
	long lOffset;

};


template<class TRecType>
bool MARCBuffer<TRecType>::Read(TRecType & rec)
{
	if(lOffset >= lLength)
		return false;

	if(rec.Parse(buf + lOffset, lLength - lOffset))
	{
	lOffset += rec.GetRecordLength();
	return true;
	}

	return false;
}


template<class TRecType>
MARCBuffer<TRecType>::MARCBuffer()
{
	buf = 0;
	lLength = 0;
	lOffset = 0;
}

template<class TRecType>
MARCBuffer<TRecType>::~MARCBuffer()
{
	Clear();
}

template<class TRecType>
MARCBuffer<TRecType>::MARCBuffer(const MARCBuffer & mb)
{
	buf = 0;
	lLength = 0;
	lOffset = 0;

	if(mb.buf)
	{
	lLength = mb.lLength;
	lOffset = mb.lOffset;

	buf = new char [lLength];
	memcpy(buf, mb.buf, lLength);
	}
}




template<class TRecType>
void MARCBuffer<TRecType>::Reset()
{
	lOffset = 0;
}

template<class TRecType>
void MARCBuffer<TRecType>::Clear()
{
	if(buf) delete [] buf;

	buf = 0;
	lLength = 0;
	lOffset = 0;
}

template<class TRecType>
MARCBuffer<TRecType> & MARCBuffer<TRecType>::operator=(const MARCBuffer & mb)
{
	if(this != &mb)
	{
	Clear();
	if(mb.buf)
		{
		lLength = mb.lLength;
		lOffset = mb.lOffset;
		buf = new char [lLength];
		memcpy(buf, mb.buf, lLength);
		}
	}

	return *this;
}

template<class TRecType>
MARCBuffer<TRecType> MARCBuffer<TRecType>::operator+(const MARCBuffer & mb)
{
	MARCBuffer tmp;

	tmp.lLength = lLength + mb.lLength;
	tmp.lOffset = 0;

	tmp.buf = new char [tmp.lLength];

	if(buf)	memcpy(tmp.buf, buf, lLength);

	if(mb.buf) memcpy(tmp.buf + lLength, mb.buf, mb.lLength);

	return tmp;
}

template<class TRecType>
bool MARCBuffer<TRecType>::LoadFromFile(const char * szFileName)
{
	lOffset = 0;
	fstream file;
	file.open(szFileName, ios::in);
	if(!file) return false;
	file.seekg(0, ios::end);
	lLength = file.tellg();
	file.seekg(0);
	if(buf) delete [] buf;
	buf = new char [lLength];
	file.read((char *)buf, lLength);
	file.close();
	return true;
}

template<class TRecType>
bool MARCBuffer<TRecType>::LoadFromMem(const char * buffer, long lSize)
{
	if(buf) delete [] buf;

	lLength = lSize;
	lOffset = 0;

	buf = new char [lLength];
	memcpy(buf, buffer, lLength);
}


template<class TRecType>
bool  MARCBuffer<TRecType>::ReadFromFile(TRecType & rec, char * szFileName, long lBufSize)
{
/*	if(szFileName)
	{
	fstream file;
	file.open(szFileName, ios::in);

	if(!file) return false;

	}

	file.seekg(0, ios::end);
	lLength = file.tellg();
	file.seekg(0);
	buf = new unsigned char [lLength];
	file.read((char *)buf, lLength);
	file.close();
*/
	return true;
}


#endif