/****************************************************
	MARC Template Library 

  	Version 0.6.1 Released 12 June 2002
    Version 0.6 - Released 10 June 2002
	  Version 0.5 - Not Released 1st June 2002
    Version 0.4.5 - Released 5th April 2002
	Version 0.4 - Released 28th March 2002
	Version 0.3 - Released 18th January 2002
	Version 0.2 - Released 14th January 2002
	Version 0.1 - Released 8th January 2002


	Copyright Mark Basedow 2002.
	Email: markbasedow@bigpond.com

	Please read licence.txt that is
	distributed with this file.

  ***************************************************/

#ifndef _MARCRECORD_H
#define _MARCRECORD_H


#include <string>
#include "marcutil.h"
#include <vector>
#include "marcfield.h"
#include "marcdatafield.h"
#include "marccontrolfield.h"
#include "marcstring.h"

using namespace std;

/*
template <class TLeader, class TDir> class MARCRecord;


template<class TLeader, class TDir>
class marc_iterator 
{
public:
		marc_iterator() {}
//		marc_iterator(MARCRecord<TLeader, TDir> * pRec, IternIter ii) { p = pRec; pos = ii; }

		MARCField & operator*()
		{
		mf.Parse((MARCTag)pos->first, p->buf + pos->second.iStartPos + p->leader.baseAddressOfData, pos->second.iLength);
		return mf;
		}

		MARCField * operator->() 
		{
		mf.Parse((MARCTag)pos->first, p->buf + pos->second.iStartPos + p->leader.baseAddressOfData, pos->second.iLength);
		return &mf;
		}

		marc_iterator & operator++()
		{
		++pos;
		return *this; 
		}

		marc_iterator operator=(marc_iterator & i)
		{
		p = i.p; pos = i.pos; mf = i.mf;
		return *this; 
		}

		bool operator==(const marc_iterator & i) 
		{ return (pos == i.pos); }

		bool operator!=(const marc_iterator & i) 
		{ return (pos != i.pos); }


	private:
		MARCRecord<TLeader, TDir> * p;
		multimap<MARCTag, TDir>::iterator pos;
		MARCField mf;
};

*/

class MARCField;


	class MARCFieldPtr
	{
	public:
		MARCFieldPtr(MARCField * ptr = 0) : p(ptr) { }
		~MARCFieldPtr() { if(p) delete p; }
	
		MARCFieldPtr(const MARCFieldPtr & mfp)
		{
			p = mfp() ? mfp()->Clone() : 0;
		}

		MARCField * NewField(MARCTag & mt, char * szBuf, int iLen, bool bOwnsMemory = true)
		{
		if(p) delete p;

		if(mt.IsControl())
			p = new MARCControlField;
		else
			p = new MARCDataField;
		p->Parse(mt, szBuf, iLen);
		return p;
		}

		MARCFieldPtr & operator=(const MARCFieldPtr & mfp)
		{
		delete p;
        p = mfp()->Clone();
		return *this;
		}

		bool operator<(const MARCFieldPtr & mfp)
		{
		return p->Tag() < mfp.p->Tag();
		}

/*		MARCField * operator=(MARCDataField & mdf)
		{
		if(p) delete p;
		p = new MARCDataField;
		*p = mdf;
		return p;
		}

		MARCField * operator=(MARCControlField & mcf)
		{
		if(p) delete p;
		p = new MARCControlField;
		*p = mcf;
		return p;
		}*/

		MARCField * operator()() const { return p; }
	//	MARCField * GetField() { return p; }

	protected:
		MARCField * p;
	};




template<class TLeader, class TDir>  
class MARCRecord 
{
public:


	MARCRecord();
	virtual ~MARCRecord();
	MARCRecord(const MARCRecord<TLeader, TDir> & mr);
		
	bool Parse(char * szBuf, int iLen);
	
	TLeader & GetLeader();
	
	//Basic database like api to move through fields
	void MoveFirst();
	void MoveNext();
	bool IsEnd();
	bool FindFirst(const char * szTag);
	bool FindNext(const char * szTag);

	MARCTag * Tag();
	MARCField * Field();

	bool RemoveField();
	void AddField(MARCField * mf);

	void clear();

	string GetTitleString(bool bDiscardNonFilingChars = false, char * szTag = "245");
//	wstring GetTitleWString(char * szTag = "245");

	string GetString(const char * szTag, const char * szSubFields = NULL);

	MARCRecord<TLeader, TDir> & operator=(const MARCRecord<TLeader, TDir> & rec);
	bool operator<(const MARCRecord<TLeader, TDir> & rec) const;

	
	int GetRecordLength();
	bool IsValid() { return bValid; }
	void SetValid(bool b) { bValid = b; }

//	MARC_String Data();
//	const char * GetBuffer() const { return (const char *)buf; } 
//	int GetBufferLength() const { return iLength; }

	friend ostream & operator<<(ostream & os, const MARCRecord<TLeader, TDir> & rec);
	friend istream & operator>>(istream & is, MARCRecord<TLeader, TDir> & rec);


	// Define iterators to access fields

//	typedef marc_iterator<TLeader, TDir, multimap<MARCTag, TDir>::iterator > iterator;

//	friend class marc_iterator<TLeader, TDir, multimap<MARCTag, TDir>::iterator >;
 
	
//	iterator begin();
//	iterator end();

protected:
//	int iLength;
//	unsigned char * buf;
//	vector<MARCField>::iterator pos;
//	vector<MARCField> fields;

	vector<MARCFieldPtr>::iterator pos;
	vector<MARCFieldPtr> fields;
 
	TLeader leader;

	bool bValid;
};

template<class TLeader, class TDir>
MARCRecord<TLeader, TDir>::MARCRecord()
{

}


template<class TLeader, class TDir>
MARCRecord<TLeader, TDir>::~MARCRecord()
{

}



template<class TLeader, class TDir>
MARCRecord<TLeader, TDir>::MARCRecord(const MARCRecord<TLeader, TDir> & mr)
{
	bValid = mr.bValid;
	leader = mr.leader;
	fields = mr.fields;
}

template<class TLeader, class TDir>
bool MARCRecord<TLeader, TDir>::Parse(char * szBuf, int iLen)
{
	fields.clear();

	bValid = true;

	if(!leader.Parse(szBuf, iLen)) return false;
	int iLength = leader.GetRecLength();

	if(MARC_RT != (int)szBuf[iLength - 1]) return false;

	TDir dir;
	MARCFieldPtr fld;
	int iOffset = MARC_LEADER_LENGTH;
	int iBaseAddressOfData = leader.GetBaseAddressOfData();

	while(iOffset < iBaseAddressOfData - 1)
	{
	dir.Parse(szBuf + iOffset, MARC_DIR_LENGTH);
	fld.NewField(dir.mTag, szBuf + leader.GetBaseAddressOfData() + dir.iStartPos, dir.iLength);
	fields.push_back(fld);
	
	iOffset += MARC_DIR_LENGTH;
	}

	MoveFirst();

	return true;
}


template<class TLeader, class TDir>
void MARCRecord<TLeader, TDir>::clear()
{
	fields.clear();
	bValid = false;
}

template<class TLeader, class TDir>
MARCField * MARCRecord<TLeader, TDir>::Field()
{
	return (*pos)();
}

template<class TLeader, class TDir>
MARCTag * MARCRecord<TLeader, TDir>::Tag()
{
	return &((*pos)()->Tag());
}

/*
template<class TLeader, class TDir>
MARC_String MARCRecord<TLeader, TDir>::Data()
{
	MARC_String str;
	str.Init((char *)buf, iLength);
	return str;
}
*/
template<class TLeader, class TDir>
inline void MARCRecord<TLeader, TDir>::MoveFirst()
{
	pos = fields.begin();
}


template<class TLeader, class TDir>
inline void MARCRecord<TLeader, TDir>::MoveNext()
{
	++pos;
}

template<class TLeader, class TDir>
inline bool MARCRecord<TLeader, TDir>::IsEnd()
{
	return (pos != fields.end()) ?  false : true;
}

template<class TLeader, class TDir>
bool MARCRecord<TLeader, TDir>::FindFirst(const char * szTag)
{
	string str = szTag;

	for(; pos != fields.end(); ++pos)
	{
	if((*pos)()->Tag() == str.c_str())
		return true;
	}
	return false;
}

template<class TLeader, class TDir>
bool MARCRecord<TLeader, TDir>::FindNext(const char * szTag)
{
	string str = szTag;

	for(++pos; pos != fields.end(); ++pos)
	{
	if(pos->mTag.c_str() == str)
		return true;
	}
	return false;
}


template<class TLeader, class TDir>
bool MARCRecord<TLeader, TDir>::RemoveField()
{
	if(pos != fields.end())
	{
	pos = fields.erase(pos);
	return true;
	}

	return false;
}

template<class TLeader, class TDir>
void MARCRecord<TLeader, TDir>::AddField(MARCField * mf)
{
	MARCFieldPtr fld(mf);
	fields.push_back(fld);
}

template<class TLeader, class TDir>
MARCRecord<TLeader, TDir> & MARCRecord<TLeader, TDir>::operator=(const MARCRecord<TLeader, TDir> & rec)
{
	if(this != &rec)
	{
	bValid = rec.bValid;
	leader = rec.leader;
	fields = rec.fields;
	}

	return *this;
}

template<class TLeader, class TDir>
string MARCRecord<TLeader, TDir>::GetTitleString(bool bDiscardNonFilingChars, char * szTag)
{
	MoveFirst();

	if(FindFirst(szTag))
	{
	MARCDataField * pdf = (MARCDataField *)Field();

	char n = pdf->GetIndicator(ind2);

	if(bDiscardNonFilingChars)
	{
		if(isdigit(n)) 
		{
			return pdf->FormatString(n - 48 + 4);
		}
		else
			return pdf->FormatString();
	}
	else
	{
		return pdf->FormatString();
	}
	}

	return string();
}

template<class TLeader, class TDir>
string MARCRecord<TLeader, TDir>::GetString(const char * szTag, const char * szSubFields)
{
	string str;

	MoveFirst();

	if(FindFirst(szTag))
	{
	Field().GetString(str, szSubFields);
	}

	MoveFirst();
	return str;
}

/*
template<class TLeader, class TDir>
MARCRecord<TLeader, TDir>::iterator MARCRecord<TLeader, TDir>::begin()
{
	return MARCRecord<TLeader, TDir>::iterator(this, directory.begin());
}

template<class TLeader, class TDir>
MARCRecord<TLeader, TDir>::iterator MARCRecord<TLeader, TDir>::end()
{
	return MARCRecord<TLeader, TDir>::iterator(this, directory.end());
}
*/

template<class TLeader, class TDir>
bool MARCRecord<TLeader, TDir>::operator<(const MARCRecord<TLeader, TDir> & rec) const
{
	return false;
}

template<class TLeader, class TDir>
TLeader & MARCRecord<TLeader, TDir>::GetLeader()
{
	return leader;
}

template<class TLeader, class TDir>
int MARCRecord<TLeader, TDir>::GetRecordLength()
{
	return leader.GetRecLength();
}



template<class TLeader, class TDir>
ostream & operator<<(ostream & os, const MARCRecord<TLeader, TDir> & rec)
{
	//os.write(rec.GetBuffer(), rec.GetBufferLength());

	return os;
}


template<class TLeader, class TDir>
istream & operator>>(istream & is, MARCRecord<TLeader, TDir> & rec)
{
	string str;
	getline(is, str, '\x1d'); 
	str += "\x1d";
	rec.Parse((char *)str.c_str(), str.length());
	return is;
}


#endif
