/****************************************************
	MARC Template Library 

    Version 0.6.1 - Released 12 June 2002
	 Version 0.6 - Released 10 June 2002
	   Version 0.5 - Not Released 1st June 2002
    Version 0.4.5 - Released 5th April 2002
	Version 0.4 - Released 28th March 2002
  Version 0.3 - Released 18th January 2002
  Version 0.2 - Released 14th January 2002
  Version 0.1 - Released 8th January 2002


	Copyright Mark Basedow 2002.
	Email: markbasedow@bigpond.com

	Please read licence.txt that is
	distributed with this file.

  ***************************************************/


#ifndef _MARCRECORDEVENT_H
#define _MARCRECORDEVENT_H


#include <string>
#include "marcdatafield.h"
#include "marccontrolfield.h"
#include "marcutil.h"


using namespace std;


template<class TLeader, class TDir>
class MARCRecordEvent 
{
public:
	MARCRecordEvent();
	virtual ~MARCRecordEvent();
	MARCRecordEvent(const MARCRecordEvent & mr);
		
	bool Parse(char * szBuf, long lLen);
	
	bool IsValid(); 
	int GetRecordLength();

	virtual void startRecord(TLeader & leader) = 0; 
	virtual void endRecord() = 0;
    virtual void controlField(MARCControlField & mf) = 0;
    virtual bool dataField(MARCDataField & mf) = 0;
    virtual void subField(char code, MARC_String & ms) = 0;
	virtual void endDataField() = 0;

protected:
	bool bValid;
	int iLength;

};

template<class TLeader, class TDir>
MARCRecordEvent<TLeader, TDir>::MARCRecordEvent()
{
	iLength = 0;
	bValid = true;
}


template<class TLeader, class TDir>
MARCRecordEvent<TLeader, TDir>::~MARCRecordEvent()
{

}

template<class TLeader, class TDir>
MARCRecordEvent<TLeader, TDir>::MARCRecordEvent(const MARCRecordEvent<TLeader, TDir> & rec)
{
	iLength = rec.iLength;
	bValid = rec.bValid;
}

template<class TLeader, class TDir>
bool MARCRecordEvent<TLeader, TDir>::Parse(char * szBuf, long lLen)
{
	TLeader leader;

	if(!leader.Parse(szBuf, lLen)) return false;

	iLength = leader.GetRecLength();
	if(MARC_RT != (int)szBuf[iLength - 1]) return false;

	startRecord(leader);  // startRecord EVENT

	TDir dir;
	int iOffset = 24;
	int iBaseAddressOfData = leader.GetBaseAddressOfData();

	while(iOffset < iBaseAddressOfData - 1)
	{
	dir.Parse(szBuf + iOffset, 12);

	if(dir.mTag.IsControl())
	{
	MARCControlField cfld;
	cfld.Parse(dir.mTag, szBuf + iBaseAddressOfData + dir.iStartPos, dir.iLength, false);
	controlField(cfld); // controlField EVENT
	}
	else
	{
	MARCDataField dfld;
	dfld.Parse(dir.mTag, szBuf + iBaseAddressOfData + dir.iStartPos, dir.iLength, false);
	
	if(dataField(dfld)) // dataField EVENT ( if returns false skip subfields )
	{
		dfld.MoveFirst();
		while(!dfld.IsEnd())
		{
		// subField EVENT
		subField(dfld.SubFieldCode(), dfld.SubFieldString());
		dfld.MoveNext();
		}

		endDataField(); // endDataField EVENT ( skipped if return false from dataField EVENT)
	}
	}
	iOffset += 12;
	}

	endRecord();
	return true;
}

template<class TLeader, class TDir>
bool MARCRecordEvent<TLeader, TDir>::IsValid()
{
	return bValid;
}

template<class TLeader, class TDir>
int MARCRecordEvent<TLeader, TDir>::GetRecordLength()
{
	return iLength;
}





#endif