/****************************************************
	MARC Template Library 

  	Version 0.6.1 - Released 12 June 2002
  Version 0.6 Released 5 June 2002
    Version 0.5 - Not Released 1st June 2002
  Version 0.5 - Not Released 1st June 2002
  Version 0.4.5 - Released 5th April 2002
  Version 0.4 - Released 28th March 2002
  Version 0.3 - Released 18th January 2002
  Version 0.2 - Released 14th January 2002
  Version 0.1 - Released 8th January 2002


	Copyright Mark Basedow 2002.
	Email: markbasedow@bigpond.com

	Please read licence.txt that is
	distributed with this file.

  ***************************************************/

#include "marcstringbuffer.h"
#include "marc8xmlentity.h"



MARCStringBuffer::MARCStringBuffer(long lInitialSize)
{
	lLength = lInitialSize;
	buffer = new char [lLength];
	buffer[0] = 0;
	lEnd = 0;
}

MARCStringBuffer::MARCStringBuffer(const MARCStringBuffer & fsb)
{
	lLength = 0;
	lEnd = 0;
	buffer = 0;

	if(fsb.buffer)
	{
	lLength = fsb.lLength;
	lEnd = fsb.lEnd;
	buffer = new char[lLength];
	memcpy(buffer, fsb.buffer, lEnd);
	}
}



MARCStringBuffer & MARCStringBuffer::operator=(const MARCStringBuffer & fsb)
{
	if(&fsb != this) 
	{	
		delete [] buffer;
		lLength = fsb.lLength;
		lEnd = fsb.lEnd;
		buffer = new char[lLength];
		memcpy(buffer, fsb.buffer, lEnd);
	}
	
	return *this;
}


MARCStringBuffer::~MARCStringBuffer()
{
	delete [] buffer;
}


void MARCStringBuffer::Append(const char * szStr)
{
	long l = strlen(szStr);

	if((l + lEnd + 1) > (lLength))
	{
	char * tmp = buffer;

	lLength = (lLength * 2)  + l;

	buffer = new char [lLength + 1];
	memcpy(buffer, tmp, lEnd);
	buffer[lLength] = 0;
	delete [] tmp;
	}
	
	memcpy(buffer + lEnd, szStr, l  + 1);
	lEnd += l;
}

void MARCStringBuffer::Append(MARC_String & ms)
{
	long l = ms.StringLength;

	if((l + lEnd + 1) > (lLength))
	{
	char * tmp = buffer;

	lLength = (lLength * 2) + l;

	buffer = new char [lLength + 1];
	memcpy(buffer, tmp, lEnd);
	buffer[lLength] = 0;
	delete [] tmp;
	}
	
	memcpy(buffer + lEnd, ms.StringPtr, l);
	lEnd += l;
	buffer[lEnd] = 0;
}

void MARCStringBuffer::Append(const char * szStr, int iLen)
{
	long l = iLen;

	if((l + lEnd + 1) > (lLength))
	{
	char * tmp = buffer;

	lLength = (lLength * 2) + l;

	buffer = new char [lLength + 1];
	memcpy(buffer, tmp, lEnd);
	buffer[lLength] = 0;
	delete [] tmp;
	}
	
	memcpy(buffer + lEnd, szStr, l);
	lEnd += l;
	buffer[lEnd] = 0;
}

void MARCStringBuffer::Append(char c)
{
	if((lEnd + 2) > (lLength))
	{
	char * tmp = buffer;
	lLength = (lLength * 2) + 2;
	buffer = new char [lLength + 1];
	memcpy(buffer, tmp, lEnd);
	buffer[lLength] = 0;
	delete [] tmp;
	}
	
	buffer[lEnd] = c;
	buffer[lEnd + 1] = 0;
	++lEnd;
}


void MARCStringBuffer::Append(long l)
{
	if((lEnd + 15) > (lLength))
	{
	char * tmp = buffer;
	lLength = (lLength * 2) + 10;
	buffer = new char [lLength + 1];
	memcpy(buffer, tmp, lEnd);
	buffer[lLength] = 0;
	delete [] tmp;
	}

	ltoa(l, buffer + lEnd, 10);
	lEnd += strlen(buffer + lEnd);
}


void MARCStringBuffer::Append(int i)
{
	if((lEnd + 15) > (lLength))
	{
	char * tmp = buffer;
	lLength = (lLength * 2) + 10;
	buffer = new char [lLength + 1];
	memcpy(buffer, tmp, lEnd);
	buffer[lLength] = 0;
	delete [] tmp;
	}

	ltoa(i, buffer + lEnd, 10);
	lEnd += strlen(buffer + lEnd);
}



void MARCStringBuffer::Append(const MARCStringBuffer & fsb)
{
	if((fsb.lEnd + lEnd + 1) > (lLength))
	{
	char * tmp = buffer;
	lLength = (lLength * 2) + fsb.lLength;
	buffer = new char [lLength + 1];
	memcpy(buffer, tmp, lEnd);
	buffer[lLength] = 0;
	delete [] tmp;
	}

	memcpy(buffer + lEnd, fsb.buffer, fsb.lEnd);
	lEnd += fsb.lEnd;
}

void MARCStringBuffer::Append(MARCTag & mt)
{
	if((4 + lEnd) > (lLength))
	{
	char * tmp = buffer;

	lLength = (lLength * 2) + 5;

	buffer = new char [lLength + 1];
	memcpy(buffer, tmp, lEnd);
	buffer[lLength] = 0;
	delete [] tmp;
	}
	
	memcpy(buffer + lEnd, mt.c_str(), 3);
	lEnd += 3;
	buffer[lEnd] = 0;
}


void MARCStringBuffer::AppendMARC8EntityEncode(MARC_String & ms)
{
	if(((ms.StringLength * 6) + lEnd) > lLength)
	{
	char * tmp = buffer;

	lLength = (lLength * 2) + ms.StringLength;

	buffer = new char [lLength + 1];
	memcpy(buffer, tmp, lEnd);
	buffer[lLength] = 0;
	delete [] tmp;
	}

	MARC_String mt(buffer + lEnd, lLength - lEnd);
	lEnd += MARC8_XMLEntity::Encode(mt, ms);
}



MARCStringBuffer MARCStringBuffer::operator+(MARCStringBuffer & fcb)
{
	MARCStringBuffer tmp;
	tmp.Append(fcb);
	return tmp;
}


void MARCStringBuffer::Insert(const char * szStr, long lOffset)
{
	if(lOffset > lEnd) return;
	
	long l = strlen(szStr);

	if((l + lEnd + 1) > (lLength))
	{
	char * tmp = buffer;

	lLength = lLength + (l * 2);

	buffer = new char [lLength + 1];
	memcpy(buffer, tmp, lEnd);
	buffer[lLength] = 0;
	delete [] tmp;
	}
	memmove(buffer + lOffset + l, buffer + lOffset, lEnd - lOffset);
	memcpy(buffer + lOffset, szStr, l);
	lEnd += l;
}

void MARCStringBuffer::Clear(bool bReleaseMemory)
{
	lEnd = 0;
	if(bReleaseMemory)
	{
	delete [] buffer;
	buffer = 0;
	}
}

