
/****************************************************
	MARC Template Library 

    Version 0.5 - Released April 2002
    Version 0.4.5 - Released 5th April 2002
	Version 0.4 - Released 28th March 2002
	Version 0.3 - Released 18th January 2002
	Version 0.2 - Released 14th January 2002
	Version 0.1 - Released 8th January 2002


	Copyright Mark Basedow 2002.
	Email: mark_basedow@hotmail.com

	Please read licence.txt that is
	distributed with this file.

	MARC to Dublin Core

  ***************************************************/


#include <iostream>
#include <vector>
#include "../include/mtl.h"
#include <conio.h>
#include <algorithm>
#include <time.h>
#include <windows.h>


class MARCXML : public MARC21EventStream
{
public:

	MARCXML() : buffer(2000000), iNumRec(0) { }

	int iNumRec;
	clock_t start, finish;
	double  duration;


	void startStream() 
	{
	start = clock();
	buffer.Append("<?xml version=\"1.0\"?><collection xmlns=\"http://www.loc.gov/MARC21/slim\""
				" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"" 
				" xsi:schemaLocation=\"http://www.loc.gov/MARC21/slim"
				" http://www.loc.gov/standards/marcxml/schema/MARC21slim.xsd\""
				" type=\"Bibliographic\">");
	} 
	void endStream() 
	{
	buffer.Append("</collection>");
	finish = clock();
	} 

	void startRecord(MARC21_Leader & leader) 
	{
	++ iNumRec;
	buffer.Append("<record><leader>");
	buffer.Append(leader.GetBuffer());
	buffer.Append("</leader>");
	}

	void endRecord()
	{
	buffer.Append("</record>");
	}

    void controlField(MARCControlField & mf)
	{
	buffer.Append("<controlfield tag=\"");
	buffer.Append(mf.Tag());
	buffer.Append("\">");
	buffer.Append(mf.GetString());
	buffer.Append("</controlfield>");
	}

    bool dataField(MARCDataField & mf)
	{
	buffer.Append("<datafield tag=\"");
	buffer.Append(mf.Tag());
	buffer.Append("\" ind1=\"");
	buffer.Append(mf.GetIndicator(ind1));
	buffer.Append("\" ind2=\"");
	buffer.Append(mf.GetIndicator(ind2));
	buffer.Append("\">");
	return true;
	}

    void subField(char code, MARC_String & ms)
	{
	buffer.Append("<subfield code=\"");
	buffer.Append(code);
	buffer.Append("\">");
	buffer.AppendMARC8EntityEncode(ms);
	buffer.Append("</subfield>");
	}

	void endDataField()
	{
	buffer.Append("</datafield>");
	}

	MARCStringBuffer buffer;
};



int main(int argc, char* argv[])
{
	cout << "MARC To XML Version 0.1 \nCopyright Mark Basedow 2002.\nhttp://mtl.sourceforge.net";

	if(argc != 3)
	{
		cout << "\nUsage: marcxml [input file] [output file]";
		return 0;
	}

	clock_t start, finish;
	double  duration;
	start = clock();


	MARCXML mx;

	try{

	mx.Parse(argv[1]);
	

	fstream outfile;
	outfile.open(argv[2], ios::out);
	outfile.rdbuf()->sputn(mx.buffer.GetBuffer(), mx.buffer.GetBufferLength());
	outfile.close();


	} catch(...)
	{
	cout << "ERROR";
	}

	finish = clock();

	duration = (double)(finish - start) / CLOCKS_PER_SEC;
	cout << "\n" << mx.iNumRec << " records converted in " << duration << " seconds.";
	mx.duration = (double)(mx.finish - mx.start) / CLOCKS_PER_SEC;
	cout << "\nParse Time (excluding reading and writing from file): " << mx.duration << " seconds"; 

	return 0;
}

